import requests
import argparse
import yaml
import json
import os
import sys
from urllib3.exceptions import InsecureRequestWarning
from url_normalize import url_normalize

requests.packages.urllib3.disable_warnings(category=InsecureRequestWarning)

DEFAULT_SETTINGS_NAME = "settings.yml"


def populate_with_args(parser):
    parser.add_argument("-s", help="issue summary", type=str, required=True)
    parser.add_argument("-d", help="issue description", type=str, default="")
    parser.add_argument("-n", help="path to settings file in yaml", type=str, default="settings.yml")
    parser.add_argument("--sprint", help="put issue in current sprint", action="store_true")
    return parser


def read_settings(settings_path=DEFAULT_SETTINGS_NAME):
    try:
        with open(settings_path, 'r') as settings_file:
            return yaml.full_load(settings_file)
    except IOError:
        sys.exit("error loading settings file!")


def get_create_issue_data(settings, summary, description):
    return {
        'fields': {
            'issuetype': {'id': settings['issue_type']},
            'project': {'key': settings['project']},
            'summary': summary,
            'priority': {'id': str(settings['priority_id'])},
            'assignee': {'name': settings['assignee']},
            'reporter': {'name': settings['reporter']},
            'description': description,
            'labels': settings['labels']
        }
    }


def process_create_task_response(response, jira_url):
    if response.status_code != 201:
        print(f'error creating jira issue! Response is {response.json()} ({response.status_code})')
    else:
        body = response.json()
        issue_key = body['key']
        issue_jira_url = f"{jira_url}browse/{issue_key}"
        print(f"issue with key: {issue_key} has been created\n\n{issue_jira_url}\n\n")
        return issue_key


def get_active_sprint(settings):
    get_sprint_id_url = f"{settings['jira_url']}rest/agile/1.0/board/{settings['board_id']}/sprint?state=active"
    response = requests.get(url_normalize(get_sprint_id_url),
                            headers={'Content-Type': 'application/json'},
                            auth=(settings['login'], settings['password']),
                            verify=False)
    if response.status_code != 200:
        print(f'error getting active sprint! Response is {response.json()} ({response.status_code})')
    else:
        body = response.json()
        sprint_id = body['values'][0]['id']
        return sprint_id


def add_to_sprint(issue_key, settings):
    sprint_id = get_active_sprint(settings)
    add_to_sprint_url = f"{settings['jira_url']}/rest/agile/1.0/sprint/{sprint_id}/issue"
    data = {
        'issues': [issue_key]
    }
    response = requests.post(url_normalize(add_to_sprint_url),
                             data=json.dumps(data),
                             headers={'Content-Type': 'application/json'},
                             auth=(settings['login'], settings['password']),
                             verify=False)
    if response.status_code == 204:
        print(f"putting issue {issue_key} to sprint is successful!")
    else:
        print(f"putting issue {issue_key} to sprint is NOT successful!")


def create_jira_issue(summary, description, settings_path, to_sprint):
    settings = read_settings(settings_path)
    create_issue_url = f"{settings['jira_url']}/rest/api/2/issue"
    data = get_create_issue_data(settings, summary, description)
    jsoned_data = json.dumps(data)
    response = requests.post(url_normalize(create_issue_url),
                             data=jsoned_data,
                             headers={'Content-Type': 'application/json'},
                             auth=(settings['login'], settings['password']),
                             verify=False)
    issue_key = process_create_task_response(response, settings['jira_url'])
    if bool(issue_key) & to_sprint:
        add_to_sprint(issue_key, settings)


def validate_args(args):
    if not bool(args.n) and not os.path.exists(DEFAULT_SETTINGS_NAME):
        sys.exit("settings file not found!")


def main():
    parser = populate_with_args(argparse.ArgumentParser())
    args = parser.parse_args()
    validate_args(args)
    create_jira_issue(args.s, args.d, args.n, args.sprint)


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['create_jira_issue']

package_data = \
{'': ['*']}

install_requires = \
['PyYAML>=5.4.1,<6.0.0',
 'argparse>=1.4.0,<2.0.0',
 'requests>=2.25.1,<3.0.0',
 'url-normalize>=1.4.3,<2.0.0',
 'urllib3>=1.26.5,<2.0.0']

entry_points = \
{'console_scripts': ['cjiss = create_jira_issue.create_jira_issue:main']}

setup_kwargs = {
    'name': 'create-jira-issue',
    'version': '0.1.5',
    'description': 'CLI app to create issues in Jira from console',
    'long_description': '# create_jira_issue\n\n##Description\nThis command line app will help you to create issues from console\n##Usage\n```shell\ncjiss -s "Your issue summary" -d "your issue description" -n "./settings.yml" --sprint\n```\n##Arguments\n_s_ - [MANDATORY] issue summary\n\n_d_ - issue description\n\n_n_ - path to your issue settings in yaml (see below). If not provided, file with name "settings.yml" will be looked in\nthe current directory\n\n_sprint_ - issue will be added to current sprint.\n\n##Settings\n```yaml\n# fill this to make the script work\n\n# your jira url\njira_url: https://your.jira.url.com\n# login in jira\nlogin: your_login\n# pass in jira\npassword: your_pass\n# project you work in\nproject: PROJECT\n# jira issue type (epic, issue, etc) id\n# 3 stands for issue\nissue_type: 3\n# default priority id\npriority_id: 4\n# default assignee\nassignee: assignee_login\n# default assignee\nreporter: your_login\n# your jira board id\nboard_id: 20801\n# labels to add\nlabels:\n  - new\n```',
    'author': 'Dmitrii Akhmetshin',
    'author_email': 'elevation1987@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/44dw/create_jira_issue',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
